@@ -163,16 +163,22 @@ table2blasso <- function(table, thin, mprior, capm, m, n, cl)
     tl <- as.list(table)
     
     ## start with the easy scalars
-    l <- list(lpost=tl[["lpost"]], s2=tl[["s2"]], mu=tl[["mu"]],
-              m=tl[["m"]], lambda2=tl[["lambda2"]], pi=tl[["pi"]])
-
+    l <- list(lpost=tl[["lpost"]], llik=tl[["llik"]],s2=tl[["s2"]], mu=tl[["mu"]],
+              m=tl[["m"]], lambda2=tl[["lambda2"]], nu=tl[["nu"]], pi=tl[["pi"]])
+    
     ## now the vectors
     bi <- grep("beta.[0-9]+", names(table))
     l$beta <- as.matrix(table[,bi])
     ti <- grep("tau2i.[0-9]+", names(table))
     l$tau2i <- as.matrix(table[,ti])
-    l$tau2i[l$tau2i == -1] <- NA
-
+    ## could be that lasso is off, and tau2i doesn't exist
+    if(length(l$tau2i) == 0) l$tau2i <- NULL
+    else l$tau2i[l$tau2i == -1] <- NA
+    ti <- grep("omega2.[0-9]+", names(table))
+    l$omega2 <- as.matrix(table[,ti])
+    ## could be that Student-t is off, and omega2 doesn't exist
+    if(length(l$omega2) == 0) l$omega2 <- NULL
+    
     ## assign "inputs"
     l$T <- nrow(l$beta)
     l$thin <- "dynamic"
